'use strict';

// Declare app level module which depends on filters, and services
var eventApp = angular.module('eventApp',
                              ['eventApp.filters', 'eventApp.services', 'eventApp.directives',
                               'ngRoute','ajoslin.mobile-navigate','ngTouch'/*'ngMobile'*/, 'ngSanitize',
                               'LocalStorageModule', 'angular-carousel'])

    .config(function($provide, $logProvider, initConf) {

        // im Produktiv-Betrieb sollen Log-Ausgaben unterbunden werden...
        if (initConf.isProduction) {
            // debug-Level hat einen vorgesehenen Setter...
            $logProvider.debugEnabled(false);

            // die anderen Level nicht, daher im Produktiv-Fall deren Ausgabe nach noop umleiten
            $provide.decorator('$log', ['$delegate', function ($delegate) {
                $delegate.info = angular.noop;
                return $delegate;
            }]);
            $provide.decorator('$log', ['$delegate', function ($delegate) {
                $delegate.warn = angular.noop;
                return $delegate;
            }]);
            $provide.decorator('$log', ['$delegate', function ($delegate) {
                $delegate.error = angular.noop;
                return $delegate;
            }]);
        }
    })
    .config(function($httpProvider){

        /*
          http://victorblog.com/2012/12/20/make-angularjs-http-service-behave-like-jquery-ajax/
         */

      // Use x-www-form-urlencoded Content-Type
      $httpProvider.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded;charset=utf-8';

      // zusaetzlich geben wir unserer app noch einen extra-header zur identifikation mit, dass sie von uns ist
      $httpProvider.defaults.headers.post['fgstamp'] = 'fg-event-app';

      // Override $http service's default transformRequest
      $httpProvider.defaults.transformRequest = [function(data)
      {
        /**
         * The workhorse; converts an object to x-www-form-urlencoded serialization.
         * @param {Object} obj
         * @return {String}
         */
        var param = function(obj)
        {
          var query = '';
          var name, value, fullSubName, subName, subValue, innerObj, i;

          for(name in obj)
          {
            value = obj[name];

            if(value instanceof Array)
            {
              for(i=0; i<value.length; ++i)
              {
                subValue = value[i];
                fullSubName = name + '[' + i + ']';
                innerObj = {};
                innerObj[fullSubName] = subValue;
                query += param(innerObj) + '&';
              }
            }
            else if(value instanceof Object)
            {
              for(subName in value)
              {
                subValue = value[subName];
                fullSubName = name + '[' + subName + ']';
                innerObj = {};
                innerObj[fullSubName] = subValue;
                query += param(innerObj) + '&';
              }
            }
            else if(value !== undefined && value !== null)
            {
              query += encodeURIComponent(name) + '=' + encodeURIComponent(value) + '&';
            }
          }
          return query.length ? query.substr(0, query.length - 1) : query;
        };

        return angular.isObject(data) && String(data) !== '[object File]' ? param(data) : data;
      }];
    })
    .config(function ($compileProvider){
            // funktion noch unklar
        //$compileProvider.urlSanitizationWhitelist(/^\s*(https?|ftp|mailto|file|tel):/);
    })
    .factory('tokenInterceptor', function($rootScope, $q, $log, tokenService, authService, utilService, initConf) {
        var requestInterceptor = {
            request: function(config) {

                // gehen wir an den Server oder ist es nur ein local Page-Request
                if(config.url.indexOf('http') == 0) {

                    config.data.referer = 'mobile';

                    var av = initConf.appVersion;
                    config.data.v = av.major+'.'+av.minor+'.'+av.patch;

                    var d = $q.defer();

                    tokenService.getToken(config.isPriority).then(
                        function(token) {

                            $log.debug('gespeicherten Token ('+(token!=null ? token.name+' : '+token.value : 'none' )+') verwenden');

                            config.data[token.name] = token.value;

                            d.resolve(config);

                        }, function() {

                            $log.debug('KEIN Token vorhanden');

                            d.resolve(config);
                        }
                    );

                    return d.promise;
                }
                return config;
            },
            requestError: function(rejection) {
                $log.debug(rejection);
            },
            response: function(response) {

                if(response && response.data && response.data.status === "OK") {
                    if(response.data.token) {

                        $log.debug('neuen Token (' + response.data.token + ') vom Server erhalten und gespeichert');

                        tokenService.setNewToken(response.data.token);
                    }
                    var evBroadcasted = false;
                    if(response.data.appMinVersion) {
                        evBroadcasted = utilService.checkAppVersion(response.data.appMinVersion, 'appOutdated');
                    }
                    if(!evBroadcasted && response.data.appCurrentVersion) {
                        evBroadcasted = utilService.checkAppVersion(response.data.appCurrentVersion, 'appNewVersion');
                    }

                } else if (response.status === 401 || response.status === 500) {
                    // ausloggen, da nicht authorisiert
                    $log.debug('response: Server akzeptiert den Token nicht (mehr), User wird ausgeloggt...');
                    //authService.logout();
                    $rootScope.$broadcast('checkLogOutUser');

                    return $q.reject(response);
                }
                return response;
            },
            responseError: function(rejection) {

                // ausloggen, da nicht authorisiert
                $log.debug('responseError: Server akzeptiert den Token nicht (mehr), User wird ausgeloggt...');
                //authService.logout();
                $rootScope.$broadcast('checkLogOutUser');

                return $q.reject(rejection);
            }
        };

        return requestInterceptor;
    })
    .config(['$routeProvider', '$httpProvider', function($routeProvider, $httpProvider) {
        $routeProvider.when('/',                              {templateUrl: 'pages/partials/dashboard.html'/*,          controller: 'DashboardController'*/});
        $routeProvider.when('/dashboard',                     {templateUrl: 'pages/partials/dashboard.html'/*,      controller: 'DashboardController'*/});
        $routeProvider.when('/sectionEntries/:type',          {templateUrl: 'pages/partials/sectionEntries.html'/*, controller: 'SectionEntriesController'*/});
        $routeProvider.when('/sectionEntriesWithEventIdent/:type/:eventIdent',     {templateUrl: 'pages/partials/sectionEntries.html'/*, controller: 'SectionEntriesController'*/});
        $routeProvider.when('/changeState/:eventIdent',       {templateUrl: 'pages/partials/changeState.html'/*,   controller: 'ChangeStateController'*/});
        $routeProvider.when('/changeState2/:eventIdent',      {templateUrl: 'pages/partials/changeState2.html'/*,   controller: 'ChangeStateController'*/});
        $routeProvider.when('/changeState3/:eventIdent',      {templateUrl: 'pages/partials/changeState3.html'/*,   controller: 'ChangeStateController'*/});
        $routeProvider.when('/changeState4/:eventIdent',      {templateUrl: 'pages/partials/changeState4.html'/*,   controller: 'ChangeStateController'*/});
        $routeProvider.when('/changeState5/:eventIdent',      {templateUrl: 'pages/partials/changeState5.html'/*,   controller: 'ChangeStateController'*/});
        $routeProvider.when('/details/:eventIdent',           {templateUrl: 'pages/partials/eventDetails.html'/*,   controller: 'EventDetailsController'*/});
        $routeProvider.when('/eventLongText/e/:eventIdent/c/:contentIdent',        {templateUrl: 'pages/partials/eventLongText.html'/*,  controller: 'EventDetailsController'*/});
        $routeProvider.when('/eventDatum/:eventIdent',        {templateUrl: 'pages/partials/eventDatum.html'/*,  controller: 'EventDetailsController'*/});
        $routeProvider.when('/eventContent/e/:eventIdent/c/:contentIdent',         {templateUrl: 'pages/partials/eventContent.html'/*,  controller: 'EventDetailsController'*/});
        $routeProvider.when('/eventContentLink/e/:eventIdent/c/:contentIdent',     {templateUrl: 'pages/partials/eventContentLink.html'/*,  controller: 'EventDetailsController'*/});
        $routeProvider.when('/eventAnsprechpartner/e/:eventIdent/c/:contentIdent', {templateUrl: 'pages/partials/eventAnsprechpartner.html'/*,  controller: 'EventDetailsController'*/});
        $routeProvider.when('/eventLocation/:eventIdent',     {templateUrl: 'pages/partials/eventLocation.html'/*,  controller: 'EventDetailsController'*/});
        $routeProvider.when('/eventLocation/e/:eventIdent/c/:contentIdent',        {templateUrl: 'pages/partials/eventLocation.html'/*,  controller: 'EventDetailsController'*/});
        $routeProvider.when('/eventWetter/e/:eventIdent/c/:contentIdent',          {templateUrl: 'pages/partials/eventWetter.html'/*,  controller: 'EventDetailsController'*/});
        $routeProvider.when('/eventFeedback/:eventIdent',     {templateUrl: 'pages/partials/eventFeedback.html'/*,  controller: 'EventDetailsController'*/});
        $routeProvider.when('/eventVideoContent/e/:eventIdent/c/:contentIdent',    {templateUrl: 'pages/partials/eventVideo.html'/*,  controller: 'EventDetailsController'*/});
        $routeProvider.when('/eventImageGallery/:eventIdent', {templateUrl: 'pages/partials/eventImageGallery.html'/*,  controller: 'EventDetailsController'*/});
        $routeProvider.when('/contact',                       {templateUrl: 'pages/partials/contact.html'/*,        controller: 'ContactController'*/});
        $routeProvider.when('/support',                       {templateUrl: 'pages/partials/support.html',        controller: 'SupportController'});
        $routeProvider.when('/content/:contentIdent',         {templateUrl: 'pages/partials/content.html',        controller: 'ContentController'});
        $routeProvider.when('/login',                         {templateUrl: 'pages/partials/login.html' /*,          controller: 'LoginController'*/});
        $routeProvider.when('/logout',                        {templateUrl: 'pages/partials/login.html',          controller: 'LogoutController'});

        $routeProvider.otherwise({redirectTo: '/dashboard'});


        $httpProvider.interceptors.push('tokenInterceptor');

    }])
    .constant('initConf', {
        isProduction : true,
        appVersion : {major: 1, 'minor': 2, 'patch': 1, buildDate: '20160429120000'},

        serverURL : 'https://www.forumgelb.de',

        eventReloadIntervall          : 1000 * 60 * 3, // 3 Minuten
        eventQuestionsReloadIntervall : 1000 * 60 * 3, // 3 Minuten
        contentReloadIntervall        : 1000 * 60 * 3, // 3 Minuten
        kamReloadIntervall            : 1000 * 60 * 60 * 24, // 1 Tag

        minTokenCount                 : 2, // 2, damit wir noch Puffer fuer Priority haben
        tokenFillupThreshold          : 10,
        tokenResponseTimeoutThreshold : 30 * 1000
    })
    .run(function ($rootScope, authService, tokenFillUpService, eventService, localStorageService) {
        //                                  tokenFillUpService hier injecten, damit gleich initialisiert wird statt lazy
        // Migration von 1.0.1 auf 1.1.0 altes Auth-Object entfernen
        localStorageService.remove('auth');

        document.addEventListener("pause", onPause, false);

        function onPause() {
            // Handle the pause event
            if(navigator.splashscreen) navigator.splashscreen.show();
            // ? laesst uns eigentlich das pause-event genug Zeit, einen evtl. Logout "durchzuziehen" ?
            //authService.checkStayLoggedIn();
        }

        document.addEventListener("resume", onResume, false);

        function onResume() {
            // ist zwingend notwendig, damit alle Angular-spezifischen Events funktionieren
            // (im Spziellen kommt sonst kein $routeChangeSuccess und damit keine Wechsel zur Login-Seite
            $rootScope.$apply(function() {
                // Handle the resume event
                if (navigator.splashscreen) navigator.splashscreen.hide();

                // sicherstellen, dass wir bei Wiederkehr noch berechtigt sind, weiterzumachen...
                authService.checkStayLoggedIn();

                if (authService.isLoggedIn() && !eventService.getEventsLoaded()) {
                    eventService.loadEvents();
                }
            });
        }

    });
