"use strict";

/* Controllers */

function NavigationController($rootScope, $scope, $route, $log, authService, tokenService, navSvc, eventService, notificationService) {

    if(!authService.isLoggedIn ()) {
        // zur LoginSeite
        navSvc.slidePage('/login','none');
    } else if (!authService.hasSeenSupport()) {
        navSvc.slidePage('/support','modal');
    }

    $rootScope.$on('checkLogOutUser', function(event){
        if(tokenService.tokensEmpty()) {
            $log.debug('user ist nicht mehr eingeloggt');
            $scope.slidePage('/logout','none');
        }
    });
    $rootScope.$on('logOutUser', function(event){

        $scope.slidePage('/logout','none');
    });
    $rootScope.$on('appOutdated', function(event){
        $log.debug('app version wird nicht mehr vom Server unterstuetzt');
        notificationService.alert('Sie verwenden eine veraltete Version dieser App. Diese wird nicht mehr unterstützt. Bitte aktualisieren Sie die forum gelb App, da es sonst zu Fehlern kommen kann.',null,'Hinweis');
    });
    $rootScope.$on('appNewVersion', function(event){
        $log.debug('server meldet neue app version verfuegbar');
        notificationService.alert('Es gibt eine neue Version dieser App. Bitte aktualisieren Sie die forum gelb App.',null,'Hinweis');
    });

    $rootScope.isShowNavButtons = true;
    $rootScope.isShowMenu = false;
    $rootScope.isShowSearch = false;

    $scope.slidePage = function (path,type) {
        navSvc.slidePage(path,type);
    };
    $scope.back = function () {
        navSvc.back();
    };
    $scope.logout = function () {
        notificationService.confirm('Wenn Sie sich jetzt ausloggen, müssen Sie sich beim Öffnen der App erneut mit Ihren Zugangsdaten anmelden.',
                                    function(iWant){
                                        if(iWant == 1) { // erster Button ist der 'Ausloggen'-Button
                                            $scope.slidePage('/logout','none');
                                        }
                                    }, 'Information', ['Fortfahren','Abbrechen']);
    }
    $scope.showNav = function () {
        return authService.isLoggedIn();
    };
    $scope.getUserName = function () {
        return authService.getUserName();
    };
    $scope.isDashboard = function () {
        return ($route.current.originalPath == '/dashboard') ? 'active' : '';
    };
    $scope.isAllEvents = function () {
        return ($route.current.params.type == 'allEvents') ? 'active' : '';
    };
    $scope.isMyEvents  = function () {
        return ($route.current.params.type == 'myEvents') ? 'active' : '';
    };
    $scope.isLookbacks = function () {
        return ($route.current.params.type == 'lookbacks') ? 'active' : '';
    };

    $scope.switchMenu = function () {
        $rootScope.isShowMenu = ! $rootScope.isShowMenu;
    };
    $scope.showMenuBySwipe = function () {
        if(!$scope.isShowBack && !$scope.isShowMenu) $scope.switchMenu();
    };
    $scope.hideMenuBySwipe = function () {
        if($scope.isShowMenu) $scope.switchMenu();
    };
    $scope.switchSearch = function () {
        $scope.searchMsg = null;
        $scope.webcode.text = '';
        $rootScope.isShowSearch = !$rootScope.isShowSearch;
    };

    $rootScope.isNavMenuOpen = function () {
        return $rootScope.isShowMenu;
    }
    $rootScope.isSearchMenuOpen = function () {
        return $rootScope.isShowSearch;
    }
    $scope.hasSeenSupport = authService.hasSeenSupport();
    $rootScope.$on('hasSeenSupport', function(){
        $scope.hasSeenSupport = authService.hasSeenSupport();
    });

    // wir brauchen im Scope des Controllers ein Objekt, auf das wir im Input-Feld per ng-model zugreifen koennen.
    // Wuerden wir nur einen "einfache" Variable per ng-model erzeugen, waere diese im Scope des includes, der
    // Controller ist aber "ueber" dem include definiert... Somit koente der Controller die "einfache" Variable
    // nicht "sehen"
    $scope.webcode = {text:''};

    $scope.searchEvent = function () {
        eventService.getByWebcode($scope.webcode.text).then(function(eventIdent){
            if(eventIdent == null) {
                $scope.searchMsg = 'Kein Ergebnis gefunden.';
            } else {
                $scope.searchMsg = null;
                $scope.webcode.text = '';
                $rootScope.isShowSearch = false;
                navSvc.slidePage('/details/'+eventIdent);
            }
        });
    };

}

function LoginController($rootScope, $scope, $log, authService, loginService, navSvc, notificationService, pushNotificationService) {

    $scope.user = null;
    $rootScope.isShowBack = false;
    $scope.showLoading = false;

    navSvc.eraseHistory();

    $scope.user = {stayLoggedIn : false};

    $scope.showNotification = function () {
        if(!$scope.user.stayLoggedIn) {
            notificationService.confirm('Wenn Sie dauerhaft eingeloggt bleiben, besteht die Möglichkeit, dass andere Personen diese App ohne Nachfrage einsehen.',

                    function (iWant) {
                        if (iWant == 1) { // der 1-Button ist der 'Doch nicht'-Button
                            $scope.user.stayLoggedIn = false;
                        }
                    }, 'Information', ['Abbrechen', 'Zustimmen']);
        }
    };

    $scope.submit = function() {

        if($scope.user == null){
            notificationService.alert('Bitte geben Sie einen Benutzernamen / E-Mail-Adresse und ein Passwort an.',null,'Login');
        }
        else if($scope.user.username == null) {
            notificationService.alert('Bitte geben Sie einen Benutzernamen / E-Mail-Adresse an.',null,'Login');
        }
        else if($scope.user.password == null) {
            notificationService.alert('Bitte geben Sie ein Passwort an.',null,'Login');
        }
        else
        {
            $scope.showLoading = true;
            loginService.authorize($scope.user).then(
              function(result) { // success
                  if(result.auth) {

                      if($scope.user.stayLoggedIn) {

                          authService.storeAuthPermanent();

/*
                          notificationService.confirm('Wenn Sie dauerhaft eingeloggt bleiben, können Personen, die Ihr Smartphone nutzen, diese App ohne Nachfrage einsehen.',
                                                      function(iWant){
                                                          if(iWant == 2) { // der 2-Button ist der 'Abbrechen'-Button
                                                              $scope.user.stayLoggedIn = false;
                                                          }
                                                          if(iWant == 1) { // der 1-Button ist der 'Verstanden'-Button
                                                              authService.storeAuthPermanent();
                                                          }
                                                      }, 'Hinweis', ['Verstanden','Abbrechen']);
*/
                      }

                      if(result.hasSeenSupport) {
                          navSvc.slidePage('/dashboard');
                      } else {
                          navSvc.slidePage('/support', 'modal');
                      }

/*
                      window.onNotificationGCM = function (e) {
                          pushNotificationService.onNotificationGCM (e);
                      }

                      if(window.plugins) {
                            var pushNotification = window.plugins.pushNotification;
                            pushNotification.register(pushNotificationService.pushNotififationSuccessHandler,
                                    pushNotificationService.pushNotififationErrorHandler,
                                    {"senderID": "1072212169274", "ecb": "onNotificationGCM"});
                      } else {
                            console.log('Jetzt wuerde das window.plugins.pushNotification angesprochen werden');
                      }
*/

                  }
                  else
                  {
                      // der else-Zweig sollte nicht auftreten, aber "sicher ist sicher"
                      notificationService.alert('Entschuldigung, es sind Probleme mit unserem Server aufgetreten.', null /*callback*/, 'Login');
                  }
                  $scope.showLoading = false;
              },
              function(result) { // error
                  notificationService.alert(result.reason, null /*callback*/, 'Login');
                  $scope.showLoading = false;
              }


            );

        }
    };

    $scope.forgottenPassword = function () {
        if($scope.user == null || $scope.user.username == null){
            notificationService.alert('Bitte geben Sie Ihren Benutzernamen oder die E-Mail-Adresse an, die Sie bei uns hinterlegt haben und wählen noch einmal "Passwort vergessen". '+
                                        'Wir senden Ihnen dann umgehend ein neues Passwort zu.',null,'Passwort vergessen');
        }
        else {
            notificationService.confirm('Wollen Sie sich ein neues Passwort an Ihre bei uns hinterlegte E-Mail-Adresse zusenden lassen?',
                                        function(iWant){
                                            if(iWant == 1) { // erster Button ist der 'Neues Passwort'-Button
                                                authService.sendNewPassword($scope.user.username).then(
                                                    function(result) { // success
                                                          if(result.send) {
                                                              notificationService.alert('Ein neues Passwort wurde an Ihre Email-Adresse verschickt.', null /*callback*/, 'Passwort vergessen');
                                                          }
                                                          else
                                                          {
                                                              notificationService.alert('Benutzername und/oder Email nicht bekannt.', null /*callback*/, 'Passwort vergessen');
                                                          }
                                                          $scope.showLoading = false;
                                                    },
                                                    function(result) { // error
                                                          notificationService.alert(result.reason, null /*callback*/, 'Passwort vergessen');
                                                          $scope.showLoading = false;
                                                    }
                                                );
                                            }
                                        }, 'Hinweis', ['Neues Passwort','Abbrechen']);

        }
    };

}

function LogoutController($rootScope, $scope, authService, kamInfoService, eventService, navSvc) {

  navSvc.eraseHistory();
  authService.logout();
  // TODO das "aufraeumen muss zentraler erfolgen... entweder durch ein "usersession-objekt" oder durch listener, die bei logout informiert werden...
  kamInfoService.resetKAMInfos();

  eventService.resetEvents ();

  $rootScope.isShowMenu = false;
  $rootScope.isShowSearch = false;
}

function DashboardController($rootScope, $scope, navSvc, eventService,notificationService) {
    $rootScope.isShowMenu = false;
    $rootScope.isShowSearch = false;
    $rootScope.isShowBack = false;

    function bindData () {
        // databinding
        $scope.myNext = eventService.getMyEvents();
        $scope.allNextEvents = eventService.getEvents();
        $scope.allNextKK = eventService.getKKEvents();
        $scope.allNextER = eventService.getEREvents();
        //$scope.lookbacks = eventService.getLookbacksByYear(); //eventService.getEventLookbacks().concat(eventService.getKKEventLookbacks());
        $scope.lookbacksEvents = eventService.getEventLookbacks();
        $scope.lookbacksKK = eventService.getKKEventLookbacks();
        $scope.lookbacksER = eventService.getEREventLookbacks();

        $scope.showMyNext =          eventService.getMyEventsLength() > 0;
        $scope.showAllNextEvents =   eventService.getEventsLength() > 0;
        $scope.showAllNextKK =       eventService.getKKEventsLength() > 0;
        $scope.showAllNextER =       eventService.getEREventsLength() > 0;
        $scope.showLookbacksEvents = eventService.getEventLookbacksLength() > 0;
        $scope.showLookbacksKK =     eventService.getKKEventLookbacksLength() > 0;
        $scope.showLookbacksER =     eventService.getEREventLookbacksLength() > 0;
    }

    $scope.showLoading = false;

    // erstmal laden
    if(eventService.getEventsLoaded()) {
        bindData();
    } else {
        $scope.showLoading = true;
        eventService.loadEvents().then(function(){
            bindData();
            $scope.showLoading = false;
        },
        function(reason) { // error
            notificationService.alert('Entschuldigen Sie bitte. Die Veranstaltungsdaten konnten gerade nicht geladen werden.',null/*callback*/,'Fehler');
            $scope.showLoading = false;
        });
    }

    $scope.showDetails = function (eventIdent, type) {
        navSvc.slidePage('/sectionEntriesWithEventIdent/'+type+'/'+eventIdent);
    }

    $scope.carouselIndex = 0;
    $scope.carouselNext = function () { if($scope.carouselIndex < 1) $scope.carouselIndex++; }
    $scope.carouselPrev = function () { if($scope.carouselIndex > 0) $scope.carouselIndex--; }
}

function SectionEntriesController($rootScope, $scope, $q, $route, $routeParams, $log, $timeout, navSvc, eventService, authService, imageService, notificationService) {
    $rootScope.isShowMenu = false;
    $rootScope.isShowSearch = false;
    $rootScope.isShowBack = false;

    $scope.type = $routeParams.type;
    $scope.carouselIndex = 0;

    var selectedEventIdent = null;
    if($routeParams.eventIdent != null) {
        selectedEventIdent = $routeParams.eventIdent;
    }

    $scope.showLoading = false;

    function addTeaserImageURLs (events) {
        var imageURLPromises = [];
        var carouselIndex = 0;
        // fuer das Carousel die Umrechnung von EventIdent auf Carousel-Array-Index machen
        angular.forEach(events, function (event, index){
            if(event.ident == selectedEventIdent) carouselIndex = index;

            var imageURLPromise = event.teaserImages != null ? imageService.loadImage(event.teaserImages[0]) : $q.when('');

            imageURLPromises.push(imageURLPromise);
            imageURLPromise.then(function (imagePath) {
                event.teaserImages != null ? $log.debug('imageURL fuer '+event.teaserImages[0]+' berechnet '+imagePath) : $log.debug('Kein(e) event.teaserImages gesetzt.');
//                $scope.events[index].eventTeaserImage = imagePath;
//                $scope.events[index].eventTeaserImageLoaded = true;
                event.eventTeaserImage = imagePath;
                event.eventTeaserImageLoaded = true;
            }, function(e){
                $log.debug('SectionEntriesController: keine imageURL fuer '+event.ident);
            });

        });
        $q.all(imageURLPromises).then(function(){
            $scope.showLoading = false;
            $scope.events = events;

            // etwas verzoegern, damit das Carousel auch bereit fuer einen anderen Index ist
            $timeout(function(){
                $scope.carouselIndex = carouselIndex;
            }, 0);

            $log.debug('alle Event-Teaser-Bilder geladen');
        }, function(){
            $scope.showLoading = false;
            $scope.events = events;

            $log.debug('NICHT alle Event-Teaser-Bilder geladen');
        });

    }

    $scope.showLoading = true;

    // erstmal laden
    if(eventService.getEventsLoaded()) {
        var events = eventService.getAllByType($routeParams.type);

        addTeaserImageURLs (events);

    } else {
        eventService.loadEvents().then(
            function() { // success

                var events = eventService.getAllByType($routeParams.type);

                addTeaserImageURLs (events);
            },
            function(reason) { // error
                notificationService.alert('Entschuldigen Sie bitte. Die Veranstaltungsdaten konnten nicht geladen werden.',null/*callback*/,'Fehler');
                $scope.showLoading = false;
            }
        );
    }

    //var routeWithEventIdent = $route.current.originalPath.indexOf('/sectionEntriesWithEventIdent') > -1;

    $scope.showDetails = function (eventIdent) {
        var newRoute = '/sectionEntriesWithEventIdent/'+$routeParams.type+'/'+eventIdent;
        navSvc.slidePage('/details/'+eventIdent, null, newRoute);
    }

    $scope.signIn = function (eventIdent) {
        var newRoute = '/sectionEntriesWithEventIdent/'+$routeParams.type+'/'+eventIdent;
        navSvc.slidePage('/changeState/'+eventIdent, null, newRoute);
    }
    $scope.signOff = function (eventIdent) {
        var newRoute = '/sectionEntriesWithEventIdent/'+$routeParams.type+'/'+eventIdent;
        navSvc.slidePage('/changeState4/'+eventIdent, null, newRoute);
    }

    $scope.carouselNext = function () {
        if($scope.carouselIndex < $scope.events.length-1) $scope.carouselIndex++;
    }
    $scope.carouselPrev = function () {
        if($scope.carouselIndex > 0) $scope.carouselIndex--;
    }

}

function EventDetailsController($rootScope, $scope, $sce, navSvc, $routeParams, eventService, contentService, imageService, imageTokenService, notificationService, feedbackService, initConf) {

    $scope.showLoading = true;

    eventService.getByIdent($routeParams.eventIdent).then (function(myEvent){

        if(myEvent != null && myEvent.teaserImages != null) {
            //$scope.eventTeaserImages = imageService.getTeaserImages(myEvent.teaserImages);
            imageService.loadImage(myEvent.teaserImages[0]).then(function(imagePath){
                $scope.eventTeaserImage = imagePath;
            });
        }

        $scope.event = myEvent;

        if($routeParams.contentIdent != null) {
            contentService.getByIdent($routeParams.contentIdent).then(function(content){

                    // Inlinebilder mit Server anreichern
                    if(content.text != null && content.text.indexOf('"/main/file/contentInline') > -1) {

                        imageTokenService.getImageToken().then(function(token){

                            content.text = content.text.replace(/\"\/main\/file\/contentInline\?/g, '"' + initConf.serverURL + '/main/json/teaserImage?'+token.name+'=' + token.value + '&');

                            $scope.content = content;

                        }, function(e){
                            $log.debug('kein Token für contentInline');
                        });


                    } else {
                        $scope.content = content;
                    }

                    if(content.videoURL != null) {

                        var url = content.videoURL;
                        var videoURL = initConf.serverURL + url;

                        var ampIdx = url.indexOf('&');
                        if(ampIdx > -1) {
                            // fuer das video-tag brauchen wir eine trusted Resource... ohne das &duration=4:03
                            videoURL = initConf.serverURL + url.substring(0, ampIdx);
                        }

                        $scope.videoURL = $sce.trustAsResourceUrl(videoURL);
                        //myEvent.videoDuration = url.substring(ampIdx, url.length); // muss noch das duration= weggerechnet werden...

                        if(content.teaserImage != null) {
                            //$scope.eventTeaserImages = imageService.getTeaserImages(myEvent.teaserImages);
                            imageService.loadImage(content.teaserImage.ident).then(function(imagePath){
                                $scope.contentTeaserImage = imagePath;
                            });
                        }
                    }


                    $scope.showLoading = false;
                },function(reason){ // error
                    notificationService.alert('Entschuldigen Sie bitte. Der Inhalt konnte nicht geladen werden.', null, 'Fehler');
                    $scope.showLoading = false;
                });
        } else {
            // wenn kein Content zu laden ist, dann ist das Laden ansich hier beendet und wir nehmen das "loading" weg
            $scope.showLoading = false;
        }


    },function(reason){ // error
        notificationService.alert('Entschuldigen Sie bitte. Die Veranstaltungsdaten konnten gerade nicht geladen werden.', null, 'Fehler');
        $scope.showLoading = false;
    });
    $rootScope.isShowBack = true;

    $scope.isReise = function (event) {
        return eventService.isReise(event);
    };
    $scope.signIn = function (eventIdent) {
        navSvc.slidePage('/changeState/'+eventIdent);
    };
    $scope.signOff = function (eventIdent) {
        navSvc.slidePage('/changeState4/'+eventIdent);
    };

    $scope.showLongText = function (eventIdent) {
        var contentIdent = $scope.event.isLookback ? $scope.event.lookbackContentIdent : $scope.event.infoContentIdent;
        navSvc.slidePage('/eventLongText/e/'+eventIdent+'/c/'+contentIdent);
    };
    $scope.showLocation = function (eventIdent) {
        if($scope.event.locationContentIdent != null) {
            navSvc.slidePage('/eventLocation/e/'+eventIdent+'/c/'+$scope.event.locationContentIdent);
        } else {
            navSvc.slidePage('/eventLocation/'+eventIdent);
        }
    };
    $scope.showDatum = function (eventIdent) {
        navSvc.slidePage('/eventDatum/'+eventIdent);
    };
    $scope.openMap = function (eventIdent) {

        var mapsURL = '';
        if(window.device != null && window.device.platform == 'iOS') {
            mapsURL += 'maps:q=';
        } else if (window.device != null && angular.lowercase(window.device.platform) == 'android') {
            mapsURL += 'geo:0,0?q=';
        } else {
            mapsURL += 'http://maps.google.com?q=';
        }

        eventService.getByIdent(eventIdent).then(function(event){

            if(event == null) return;

            var address='';
            address += event.street != null && event.street != '' ? event.street+',' : '';
            address += event.zipcode+' '+event.city;

            window.open (mapsURL+encodeURIComponent(address), '_system');
        },function(reason){ // error
            notificationService.alert('Entschuldigen Sie bitte. Die Veranstaltungsdaten konnten nicht geladen werden.', null, 'Fehler');
            $scope.showLoading = false;
        });
    }

    $scope.showWetter = function (eventIdent, contentIdent) {
        navSvc.slidePage('/eventWetter/e/'+eventIdent+'/c/'+contentIdent);
    }

    $scope.showCompliance = function () {
        navSvc.slidePage('/content/content_app_compliance', 'modal');
    }

    $scope.showContent = function (eventIdent, contentIdent) {
        navSvc.slidePage('/eventContent/e/'+eventIdent+'/c/'+contentIdent);
    }

    $scope.showContentLink = function (eventIdent, contentIdent) {
        navSvc.slidePage('/eventContentLink/e/'+eventIdent+'/c/'+contentIdent);
    }

    $scope.showAnsprechpartner = function (eventIdent, contentIdent) {
        navSvc.slidePage('/eventAnsprechpartner/e/'+eventIdent+'/c/'+contentIdent);
    }

    $scope.showExternalWebsite = function (url) {
        window.open(url, '_system');
        return false;
    }

	$scope.addCalEntry = function () {

        window.plugins.calendar.createEvent(
                    $scope.event.title,
                    $scope.event.location,
                    $scope.event.infoAbstract /*notes*/,
                    new Date($scope.event.startDateInMillis),
                    new Date($scope.event.endDateInMillis),
                    function(message) {
                        notificationService.alert('Der Termin wurde erfolgreich in Ihrem Kalender gespeichert.',null, '  ');
                    },
                    function(message) {
                        // bei iOS sinnvoll, bei Android gibt das Kalender-Plugin auch einen Fehler zurueck, wenn alles ok ist
                        if(window.device != null && window.device.platform == 'iOS') {
                            notificationService.alert('Bitte prüfen Sie in den Datenschutzeinstellungen, ob forum gelb auf Ihren Kalender zugreifen darf.',null, 'Fehler');
                        }
                        if(window.device != null && angular.lowercase(window.device.platform) == 'android') {
                            // hier kommen wir trotz erfolgreichem speichern hin, daher eine neutrale Meldung
                            //notificationService.alert('Daten wurden an den Kalender übermittelt.',null, ' ');
                            // auf Wunsch des Auftraggebers Text geaendert
                            notificationService.alert('Der Termin wurde erfolgreich in Ihrem Kalender gespeichert.',null, ' ');
                        }
                    }
        );
    }

    $scope.addAnsprechpartner = function () {

        if($scope.event.asp == null) return;

        // create a new contact object
        var contact = navigator.contacts.create();
        contact.displayName = $scope.event.asp.name;
        contact.nickname = $scope.event.asp.name;            // specify both to support all devices

/*
        var name = new ContactName();
        name.givenName = "Jane";
        name.familyName = "Doe";
        contact.name = name;
*/

        var phoneNumbers = [];
        phoneNumbers[0] = new ContactField('work', $scope.event.asp.phone, true); // preferred number
        phoneNumbers[1] = new ContactField('work fax', $scope.event.asp.fax, false);
        contact.phoneNumbers = phoneNumbers;

        // Email
        var emails = [];
        emails[0] = new ContactField('work', $scope.event.asp.email, true);
        contact.emails = emails;

        // save to device
        contact.save(
            function(){ // success
                notificationService.alert('Der Kontakt wurde erfolgreich gespeichert.', null, ' ');
            },
            function(message){ // error
                if(window.device != null && window.device.platform == 'iOS' && message.code == ContactError.PERMISSION_DENIED_ERROR) {
                    notificationService.alert('Bitte prüfen Sie in den Datenschutzeinstellungen, ob forum gelb auf Ihre Kontakte zugreifen darf.', null, 'Fehler');
                } else {
                    notificationService.alert('Die Kontaktdaten konnten nicht gespeichert werden.', null, 'Fehler');
                }
            });

    };
    //$scope.feedback = {text:''};
    $scope.answers = {};
    $scope.eventOverallAvg = function(event) {
        return event != null ? Math.round(event.overallAvg) : 0;
    };
    $scope.showEventFeedback = function (eventIdent) {
        navSvc.slidePage('/eventFeedback/'+eventIdent);
    };
/*
    $scope.sendEventFeedback = function () {
        $scope.showLoading = true;
        feedbackService.sendEventFeedback($scope.event.ident, $scope.feedback.text).then(
                function () { // success
                    notificationService.alert('Vielen Dank. Ihr Feedback wurde gesendet.', null, ' ');
                    $scope.feedback = {text:''};
                    $scope.showLoading = false;
                },
                function () { // error
                    notificationService.alert('Entschuldigen Sie bitte. Ihr Feedback konnte nicht gesendet werden. Bitte versuchen Sie es noch einmal.', null, 'Fehler');
                    $scope.showLoading = false;
                });
    }
*/
    $scope.sendEventRating = function () {
        $scope.showLoading = true;
        feedbackService.sendEventRating($scope.event.ident, $scope.answers).then(
                function () { // success

                    // ??? muss noch in das lokale Event-Array uebertragen werden... bzw. dort gesetzt und dann der $scope.event nochmal geladen werden
                    $scope.event.canUserRate = false;
                    var overallAnswersSum = 0;
                    var answersCount = 0;
                    angular.forEach($scope.answers, function(value, key) {
                        angular.forEach($scope.event.feedbackQuestions, function(question) {
                            if(('fba_'+question.ident) == key)
                                question.userRating = value;
                        });
                        overallAnswersSum = overallAnswersSum + parseInt(value);
                        answersCount++;
                    });
                    var overallAnswersAvg = overallAnswersSum / answersCount;
                    $scope.event.overallAvg = (($scope.event.overallAvg * $scope.event.ratingCount) + overallAnswersAvg) / ($scope.event.ratingCount + 1);
                    $scope.event.ratingCount++;

                    notificationService.alert('Vielen Dank für Ihr Feedback.', null, ' ');
                    //$scope.feedback = {text:''};
                    $scope.showLoading = false;
                },
                function () { // error
                    notificationService.alert('Entschuldigen Sie bitte. Ihr Feedback konnte nicht gesendet werden. Bitte versuchen Sie es noch einmal.', null, 'Fehler');
                    $scope.showLoading = false;
                });
    };

    $scope.showVideoContent = function (eventIdent, contentIdent) {
        if(navigator.connection != null && navigator.connection.type != Connection.WIFI) {

            notificationService.confirm('Sie sind nicht über Wifi verbunden, durch das Betrachten des Videos kann ein großes Datenvolumen entstehen.',
                                        function(iWant){
                                            if(iWant == 1) { // erster Button ist der 'Verstanden'-Button
                                                navSvc.slidePage('/eventVideoContent/e/'+eventIdent+'/c/'+contentIdent);
                                            }
                                        }, 'Hinweis', ['Verstanden','Abbrechen']);
        }
        else
            navSvc.slidePage('/eventVideoContent/e/'+eventIdent+'/c/'+contentIdent);
    };

    $scope.showImageGallery = function (eventIdent) {
            navSvc.slidePage('/eventImageGallery/'+eventIdent);
    };


}

function EventImageGalleryController($rootScope, $scope, $routeParams, $swipe, $sce, navSvc, eventService, imageGalleryService, notificationService) {
    $scope.showLoading = true;

    $scope.loadedPage = 1;
    $scope.pageSize = 0;
    $scope.imagesSize = 0;
    $scope.carouselIndex = 0;
    $scope.$watch('carouselIndex', function(newValue){
        if(newValue > ($scope.imagesSize - 3) && $scope.loadedPage < $scope.pageSize) {
            $scope.loadNextPage();
        }
    });
    $scope.showLargeImage = false;
    eventService.getByIdent($routeParams.eventIdent).then (function(myEvent){

        if(myEvent != null && myEvent.hasGallery) {
            imageGalleryService.loadImageGallery(myEvent.ident).then(function(imageGallery){
                $scope.galleryImages = imageGallery.images;
                $scope.pageSize = imageGallery.pageSize;
                $scope.imagesSize = $scope.galleryImages.length;
            });
        }

        $scope.event = myEvent;

        $scope.showLoading = false;

    },function(reason){ // error
        notificationService.alert('Entschuldigen Sie bitte. Die Bildergalerie konnte nicht geladen werden.', null, 'Fehler');
        $scope.showLoading = false;
    });
    $rootScope.isShowBack = true;

    $scope.loadNextPage = function () {
        imageGalleryService.loadImageGallery($routeParams.eventIdent, ++$scope.loadedPage).then(function(imageGallery){
            angular.forEach(imageGallery.images, function(image){
                $scope.galleryImages.push(image);
                $scope.imagesSize = $scope.galleryImages.length;
            });
            if ($scope.loadedPage >= $scope.pageSize) {
                $scope.hideLoadMoreButton = true;
                return;
            }
        });
    }
    $scope.showThumbs = false;
    $scope.switchThumbs = function () {
        $scope.showThumbs = !$scope.showThumbs;
    }

    $scope.showImage = function (image, index) {
        $scope.carouselIndex = index;
        $scope.showThumbs = false;
    }
    $scope.hideImage = function () {
        $scope.showLargeImage = false;
        $rootScope.isShowNavButtons = true;
        if(!$rootScope.$$phase) $rootScope.$digest(); // hier nur notwendig, da wir mit dem .on('click', ....) aus dem angular-system fallen
    }

    $scope.carouselNext = function () {
        if($scope.carouselIndex < $scope.galleryImages.length-1) $scope.carouselIndex++;
    }
    $scope.carouselPrev = function () {
        if($scope.carouselIndex > 0) $scope.carouselIndex--;
    }

}

function ContactController($rootScope, $scope, navSvc, kamInfoService, notificationService) {
    $rootScope.isShowMenu = false;
    $rootScope.isShowSearch = false;
    $scope.showLoading = true;

    kamInfoService.getKAMInfos().then(function(_kamInfos){
        $scope.kamInfo = _kamInfos;
        $scope.showLoading = false;
    },function(reason){ // error
        notificationService.alert('Die Kontaktdaten konnten nicht geladen werden.', null, 'Fehler');
        $scope.showLoading = false;
    });

    $scope.addKamContact = function (kam) {

        // create a new contact object
		var contact = navigator.contacts.create();
		contact.displayName = kam.name;
		contact.nickname = kam.name;            // specify both to support all devices

        var emails = [];
        emails[0] = new ContactField('work', kam.kamMail, false);
		contact.emails = emails;

		var phoneNumbers = [];
		phoneNumbers[0] = new ContactField('work', kam.kamPhone, false);
		phoneNumbers[1] = new ContactField('mobile', kam.kamMobilePhone, true); // preferred number
		phoneNumbers[2] = new ContactField('fax', kam.kamFax, false);
		contact.phoneNumbers = phoneNumbers;

        // save to device
        contact.save(
            function(){ // success
                notificationService.alert('Der Kontakt wurde erfolgreich gespeichert.', null, ' ');
            },
            function(message){ // error
                if(window.device != null && window.device.platform == 'iOS' && message.code == ContactError.PERMISSION_DENIED_ERROR) {
                    notificationService.alert('Bitte prüfen Sie in den Datenschutzeinstellungen, ob forum gelb auf Ihre Kontakte zugreifen darf.', null, 'Fehler');
                } else {
                    notificationService.alert('Die Kontaktdaten konnten nicht gespeichert werden.', null, 'Fehler');
                }
            });
    }

    $scope.addFGContact = function () {

		// create a new contact object
		var contact = navigator.contacts.create();

        contact.displayName = "forum gelb-Team";
		contact.nickname = "forum gelb-Team";            // specify both to support all devices

        var emails = [];
        emails[0] = new ContactField('work', $scope.kamInfo.mail, false);
		contact.emails = emails;

		var phoneNumbers = [];
		phoneNumbers[0] = new ContactField('work', $scope.kamInfo.phone, true);
		phoneNumbers[1] = new ContactField('fax', $scope.kamInfo.fax, false);
		contact.phoneNumbers = phoneNumbers;

        // save to device
        contact.save(
            function(){ // success
                notificationService.alert('Der Kontakt wurde erfolgreich gespeichert.', null, ' ');
            },
            function(message){ // error
                if(window.device != null && window.device.platform == 'iOS' && message.code == ContactError.PERMISSION_DENIED_ERROR) {
                    notificationService.alert('Bitte prüfen Sie in den Datenschutzeinstellungen, ob forum gelb auf Ihre Kontakte zugreifen darf.', null, 'Fehler');
                } else {
                    notificationService.alert('Die Kontaktdaten konnten nicht gespeichert werden.', null, 'Fehler');
                }
            });
	}
}

function SupportController($rootScope, $scope, authService, navSvc) {
    $rootScope.isShowMenu = false;
    $rootScope.isShowSearch = false;

    var u = authService.getUser();
    $scope.welcome = u.salutation + ' ' + (u.title != null && u.title.length > 0 ? u.title + ' ' : '') + u.lastName;

    $scope.hasSeenSupport = authService.hasSeenSupport();

    $scope.closeSupport = function () {
        authService.setHasSeenSupport();
        navSvc.slidePage('/dashboard','none');
        $rootScope.$broadcast('hasSeenSupport');
    }
}

function ContentController($rootScope, $scope, $routeParams, navSvc, authService, contentService, imageTokenService, notificationService, initConf) {
    $rootScope.isShowMenu = false;
    $rootScope.isShowSearch = false;
    $scope.content = "";
    $scope.showLoading = true;

    contentService.getByIdent($routeParams.contentIdent)
            .then(function(content){ // success
                // Inlinebilder mit Server anreichern
                if(content.text != null && content.text.indexOf('"/main/file/contentInline') > -1) {

                    imageTokenService.getImageToken().then(function(token){

                        content.text = content.text.replace(/\"\/main\/file\/contentInline\?/g, '"' + initConf.serverURL + '/main/json/teaserImage?'+token.name+'=' + token.value + '&');

                        $scope.content = content;
                        $scope.showLoading = false;

                    }, function(e){
                        $log.debug('kein Token für contentInline');
                    });


                } else {
                    $scope.content = content;
                    $scope.showLoading = false;
                }

            },function(reason){ // error
                notificationService.alert('Entschuldigen Sie bitte. Der Inhalt konnte nicht geladen werden.', null, 'Fehler');
                $scope.showLoading = false;
            });
}


function ChangeStateController($rootScope, $scope, $routeParams, $log, navSvc, authService, eventService, eventQuestionAnswerService, contentService, imageTokenService, notificationService, initConf) {
    $rootScope.isShowMenu = false;
    $rootScope.isShowSearch = false;

    $scope.showLoading = true;

    eventService.getByIdent($routeParams.eventIdent).then (function(myEvent){
        $scope.event = myEvent;

        if(myEvent.reisebedingungenContentIdent !== undefined) {
            contentService.getByIdent(myEvent.reisebedingungenContentIdent)
                    .then(function(content){ // success
                        // Inlinebilder mit Server anreichern
                        if(content.text != null && content.text.indexOf('"/main/file/contentInline') > -1) {

                            imageTokenService.getImageToken().then(function(token){

                                content.text = content.text.replace(/\"\/main\/file\/contentInline\?/g, '"' + initConf.serverURL + '/main/json/teaserImage?'+token.name+'=' + token.value + '&');

                                $scope.reisebedingungenContent = content;

                            }, function(e){
                                $log.debug('kein Token für contentInline');
                            });


                        } else {
                            $scope.reisebedingungenContent = content;
                        }

                    });
        }
    },function(reason){ // error
        notificationService.alert('Entschuldigen Sie bitte. Verbindungsprobleme.', null, 'Fehler');
        $scope.showLoading = false;
    });
    $scope.eventQuestions = null;
    $log.debug('vor eventQuestionAnswerService.getByEventIdent für ident:'+$routeParams.eventIdent);
    eventQuestionAnswerService.getByEventIdent($routeParams.eventIdent).then(function(eventQuestions){
        $scope.showLoading = false;
        $scope.eventQuestions = eventQuestions;
    },function(reason){ // error
        notificationService.alert('Entschuldigen Sie bitte. Verbindungsprobleme.', null, 'Fehler');
        $scope.showLoading = false;
    });

    $rootScope.isShowBack = true;

    $scope.isReise = eventService.isReise;

    $scope.agbChbx = false;
    $scope.showAGBRequired = false;
    if($rootScope.answers == null) $rootScope.answers = {};

    $scope.submitAnswers = function () {

        navSvc.slidePage('/changeState2/'+$scope.event.ident);
    }
    $scope.submitAGB = function () {
        if($scope.agbChbx == false) {

            $scope.showAGBRequired = true;
            //notificationService.alert('Bitte bestätigen Sie unsere AGB\'s und Teilnahmebedingungen',null,'Hinweis');
        } else {
            $scope.showAGBRequired = false;
            $scope.showLoading = true;

            // hier speichern und bei Erfolg auf die Bestaetigungsseite wechseln
            eventQuestionAnswerService.sendAnswers($scope.event.ident, $rootScope.answers).then(
                function() { // success

                    delete $rootScope.answers; // den rootScope aufraeumen

                    $scope.showLoading = false;

                    navSvc.slidePage('/changeState3/'+$scope.event.ident);
                },
                function() { // error
                    notificationService.alert('Ihre Anmeldung konnte nicht gesendet werden. Bitte versuchen Sie es erneut.',null,'Fehler');
                    $scope.showLoading = false;
            });
        }
    }
    $scope.submitSignOff = function () {
        $scope.showLoading = true;
        // hier die Abmeldung senden und bei Erfolg auf die Bestaetigungsseite wechseln
        eventQuestionAnswerService.sendSignOff($scope.event.ident).then(
            function() { // success
                $scope.showLoading = false;
                navSvc.slidePage('/changeState5/'+$scope.event.ident);
            },
            function() { // error
                notificationService.alert('Ihre Abmeldung konnte nicht gesendet werden. Bitte versuchen Sie es erneut.',null,'Fehler');
                $scope.showLoading = false;
            }
        );
    }
    $scope.backToOverview = function () {
        navSvc.slidePage('/dashbord');
    }
}
