'use strict';

/* Services */

angular.module('eventApp.services', []).value('version', '0.1');

// phonegap ready service - listens to deviceready
eventApp.factory('phonegapReady', function() {
    return function (fn) {
        var queue = [];
        var impl = function () {
            queue.push(Array.prototype.slice.call(arguments));
        };

        function onDeviceReady () {
            queue.forEach(function (args) {
                fn.apply(this, args);
            });
            impl = fn;
        }
        if (navigator.userAgent.match(/(iPhone|iPod|iPad|Android|BlackBerry|IEMobile)/)) {
            document.addEventListener('deviceready', onDeviceReady, false);
        } else {
            onDeviceReady(); //this is the desktop browser
        }


        return function () {
            return impl.apply(this, arguments);
        };
    };
});

eventApp.factory('utilService', function($rootScope, initConf) {
    return {
        checkAppVersion : function (versionToCheck, eventToBroadcast) {
            var v = versionToCheck.split('.');
            if(v.length === 2) {
                v.push('0');
            }
            var av = initConf.appVersion;
            if(parseInt(v[0]) > av.major || parseInt(v[1]) > av.minor || parseInt(v[2]) > av.patch) {
                $rootScope.$broadcast(eventToBroadcast);
                return true;
            }
            return false;
        }
    };
});

eventApp.factory('tokenFillUpService', function($rootScope, initConf, $q, $http, $timeout, $log, localStorageService, tokenService /*, authService*/) {

    var requestsRunning = 0;
    var neededTokenCount = 0;

    var fillUpTokens = function () {
        $log.debug('fillUpTokens (requestsRunning:'+requestsRunning+')');

        var tokensLength = tokenService.getTokensLength();

        neededTokenCount = initConf.tokenFillupThreshold - tokensLength;
        requestsRunning = neededTokenCount;
        $log.debug(neededTokenCount + ' Token auffuellen');
        if(neededTokenCount <= 0) {
            requestsRunning = 1; // wenn eigentlich nix zu tun ist, dann trotzdem auf 1 setzen, da requestIsBack -1 gemacht wird
            $log.debug('genug token da ('+tokensLength+')');
            // sollte es keinen Bedarf geben, dann auch die wartenden Anfragen "abarbeiten"
            requestIsBack();
        }
        while (neededTokenCount > 0) {

            $http({
                url: initConf.serverURL + "/main/json/requestAdditionalToken",
                method: "POST",
                isPriority: true,
                data: {}
            }).success(function(data, status, headers, config) {

                $log.debug('weiteren Token vom Server erhalten ('+data.imageToken+')');

                tokenService.setNewToken(data.imageToken);

                requestIsBack ();

            }).error(function(data, status, headers, config) {

                var reason = 'Keine Verbindung zum Server.';
                $log.debug('KEIN fillupToken vom Server erhalten.');

                requestIsBack ();

            });
            neededTokenCount--;
        }
    };
    var requestIsBack = function () {
        requestsRunning--;
        if(requestsRunning <= 0) { // der letzte Durchlauf soll nochmal nachschauen, ob jetzt wirklich genug da sind
            if(tokenService.isPending()) {
                $log.debug('nachfuellen bitte');
                fillUpTokens();
            }
        }
    }
    $rootScope.$on('TokenUsedEvent', function() {
        $log.info('Event TokenUsedEvent empfangen');
          fillUpTokens();

    });

    return {
        fillUp : fillUpTokens
    };
});

eventApp.factory('tokenService', function($rootScope, $q, $timeout, $log, localStorageService, initConf /*, authService nicht erlaubt, da circular...*/) {
    var TOKEN_KEY = 'token';
    var TOKEN_ARRAY_KEY = 'tokenArray';

    var minTokenCount = parseInt(initConf.minTokenCount);

    if(localStorageService.get(TOKEN_ARRAY_KEY) == null) {
        localStorageService.add(TOKEN_ARRAY_KEY, []);
    }

    if(localStorageService.get(TOKEN_KEY) != null) {
        var tokenArray = localStorageService.get(TOKEN_ARRAY_KEY);
        // migration von einem-Token auf Token-Array
        tokenArray.push(localStorageService.get(TOKEN_KEY));
        localStorageService.remove(TOKEN_KEY);

        localStorageService.add(TOKEN_ARRAY_KEY, tokenArray);
    }

    var d = null;
    var newTokenPending = false;
    var pendingPromises = [];
    var pendingPriorityPromises = [];
    var pendingTimestamp = null;
    var timeoutPromise = null;

    var timeoutFunction = function () {
        $log.info('30 sek gewartet ohne Antwort, werfen Timeout... '+pendingPromises.length+'|'+pendingPriorityPromises.length);
        while (pendingPromises.length > 0) {
            pendingPromises.shift().reject('Timeout');
        }
        while (pendingPriorityPromises.length > 0) {
            pendingPriorityPromises.shift().reject('Timeout');
        }
        newTokenPending = false;
    };

    return {
        getToken : function (isPriority) {

            d = $q.defer();

            var tokens = localStorageService.get(TOKEN_ARRAY_KEY);
            $log.debug('-> getToken (prio='+isPriority+'): ' + tokens.length + ' Token im Pool');
            if(timeoutPromise) $timeout.cancel(timeoutPromise);

            if(!newTokenPending || (isPriority && tokens.length > 0)) {
                $log.info((tokens.length-1) + '<='+minTokenCount);
                if(tokens.length - 1 <= minTokenCount) {
                    newTokenPending = true;
                }
                var token = tokens.pop();
                localStorageService.add(TOKEN_ARRAY_KEY, tokens);

                if(!isPriority) {
                    $rootScope.$broadcast('TokenUsedEvent');
                }

                $log.debug('<- getToken: ' + tokens.length + ' Token im Pool');
                d.resolve({name: 'token', value: token });


            } else {
                $log.debug('kein neuer Token vorhanden, warten auf laufende Response');

                if(isPriority) {
                    pendingPriorityPromises.push(d);
                } else {
                    pendingPromises.push(d);
                }

                pendingTimestamp = new Date();

                timeoutPromise = $timeout(timeoutFunction, initConf.tokenResponseTimeoutThreshold);
            }
            $log.info('pending '+pendingPromises.length+'|'+pendingPriorityPromises.length);

            return d.promise;
        },
        setNewToken : function (token) {

          if(timeoutPromise) $timeout.cancel(timeoutPromise);

          var tokens = localStorageService.get(TOKEN_ARRAY_KEY);

          if(pendingPriorityPromises.length > 0) {
              $log.debug('neuer Token kommt rein, erstes wartende priority defer bedienen');
              pendingPriorityPromises.shift().resolve({name: 'token', value: token});

              timeoutPromise = $timeout(timeoutFunction, initConf.tokenResponseTimeoutThreshold);
/*
          } else if (tokens.length < initConf.minTokenCount) {
              // erstmal auffuellen, damit wir spaetere Priority bedienen koennen
              tokens.push(token);
              localStorageService.add(TOKEN_ARRAY_KEY, tokens);

              $log.debug('den token pool erstmal minimal befuellen, damit noch Auffuell-Request bedient werden koennen.');

*/
          } else if(pendingPromises.length > 0) {
              $log.debug('neuer Token kommt rein, erstes wartende defer bedienen');
              pendingPromises.shift().resolve({name: 'token', value: token});

              timeoutPromise = $timeout(timeoutFunction, initConf.tokenResponseTimeoutThreshold);
          } else {

              tokens.push(token);
              localStorageService.add(TOKEN_ARRAY_KEY, tokens);

              $log.debug('kein defer mehr in der Warteschlange, setzen flag zurueck und speicher Token in Pool');
              newTokenPending = false;

          }
            $log.info('setToken: ' + tokens.length + ' Token im Pool');
            $log.info('pending '+pendingPromises.length+'|'+pendingPriorityPromises.length);
        },
        isPending : function () {
            var value = pendingPriorityPromises.length > 0 || pendingPromises.length > 0;
            $log.debug('tokenService: isPending '+value);
            return value;
        },
        tokensEmpty : function () {
            // ganz leer geht nur, wenn tokens mit isPriority=true geholt wurden oder bewusst alle Tokens entfernt wurden (logout)
            return this.getTokensLength() == 0 && pendingPriorityPromises.length == 0;
        },
        getTokensLength : function () {
            return localStorageService.get(TOKEN_ARRAY_KEY).length;
        },
        resetTokens : function () {
            localStorageService.remove(TOKEN_ARRAY_KEY);
            localStorageService.add(TOKEN_ARRAY_KEY, []);

            // noch den Timeout canceln und gleich alle wartenden Ablehnen
            if(timeoutPromise) $timeout.cancel(timeoutPromise);
            timeoutFunction();

        }
    }
});

eventApp.factory('imageTokenService', function ($rootScope, $q, $http, $log, tokenFillUpService, tokenService, localStorageService, authService, initConf) {



    return {
        getImageToken : function () {
            $log.debug('getImageToken');
            var d = null;
            d = $q.defer();

            tokenService.getToken().then(function(data){
                $log.debug('token fuer getImageToken-Anfragen erhalten');

                d.resolve(data);
            },function(reason){
                d.reject(reason);
            });

            return d.promise;
        }
    };
});

eventApp.factory('authService', function($rootScope, $q, $log, localStorageService, tokenService) {
    var AUTH_KEY = 'auth_1_1_0';

    var SUPPORT_SEEN_KEY = 'supportSeen';

    var new_user = false;

    var authUntilLeave = null;


    function _hasSeenSupport () {
        return localStorageService.get(SUPPORT_SEEN_KEY);
    }

    function getAuth () {
        return authUntilLeave != null ? authUntilLeave : localStorageService.get(AUTH_KEY);
    }

    return {
        storeAuth : function (auth) {
            authUntilLeave = auth;
            // zur Sicherheit einen evtl. fehlerhaft zurueckgebliebenen alten Auth entfernen
            localStorageService.remove(AUTH_KEY);
        },
        storeAuthPermanent : function () {
            localStorageService.add(AUTH_KEY, authUntilLeave);
        },
        isLoggedIn : function () {
          return getAuth() != null;
        },
        getUser : function () {
          return getAuth()!=null ? {
                                        title: getAuth().title,
                                        salutation: getAuth().salutation,
                                        firstName: getAuth().firstName,
                                        lastName: getAuth().lastName
                                    } : null;
        },
        isNewUser : function () {
            return false;
            /* TODO erstmal nicht nutzen, muss erst noch getestet werden und soll beim app-bauen kein Risiko darstellen */
            var _new_user = new_user;
            new_user = false; // isNewUser soll nur beim ersten Aufruf ein true uebergeben koennen

          return _new_user;
        },
        hasSeenSupport : function () {
          return _hasSeenSupport();
        },
        setHasSeenSupport : function () {
          localStorageService.add(SUPPORT_SEEN_KEY, true);
        },
        logout : function () {
          authUntilLeave = null;
          localStorageService.remove(AUTH_KEY);

          tokenService.resetTokens();

        },
        checkStayLoggedIn : function () {
            if(!localStorageService.get(AUTH_KEY)) {
                $rootScope.$broadcast('logOutUser');
            }
        }
    }
});

eventApp.factory('loginService', function($rootScope, localStorageService, $q, $http, authService, tokenService, eventService, kamInfoService, pushNotificationService, initConf) {

    var d = null;
    return {
        authorize : function (user){

            d = $q.defer();

            // alle bisherigen gespeicherten Daten abraeumen, denn es koennte sich auch ein anderer Account anmelden...
            tokenService.resetTokens();
            eventService.resetEvents();
            kamInfoService.resetKAMInfos();

            $http({
                    url: initConf.serverURL + "/main/json/login",
                    method: "POST",
                    data: {"username":user.username,"password":user.password}
                }).success(function(data, status, headers, config) {
                  if (data.status == "OK")
                  {
                    var auth = {
                                salutation: data.salutation,
                                title: data.title,
                                firstName: data.firstName,
                                lastName: data.lastName
                               };

                      authService.storeAuth(auth);

                    d.resolve({auth:true, hasSeenSupport: authService.hasSeenSupport()});

                }
                else
                {
                    d.reject({auth:false, 'reason':'Der eingegebene Benutzername und/oder das Passwort sind nicht gültig.'});
                }
            }).error(function(data, status, headers, config) {
                d.reject({auth:false, 'reason':'Verbindungsprobleme zum Server. \nZur Zeit leider kein Login möglich.'});
                //alert ("error in request to server " + status);
            });

            return d.promise;
        },
        sendNewPassword : function (userName) {
            d = $q.defer();

            $http({
                    url: initConf.serverURL + "/main/forgotPassword",
                    method: "POST",
                    data: {"username": userName}
                }).success(function(data, status, headers, config) {
                  if (data == "OK_1")
                  {
                    d.resolve({send: true});

                }
                else
                {
                    d.reject({send: false});
                }
            }).error(function(data, status, headers, config) {
                d.reject({send: false, 'reason':'Verbindungsprobleme zum Server. \nZur Zeit leider kein Login möglich.'});
                //alert ("error in request to server " + status);
            });

            return d.promise;

        }
    }
});

eventApp.factory('navSvc', function($navigate) {

    return {
        slidePage: function (path, type, replaceLastEntry) {
            $navigate.go(path,type, false, replaceLastEntry);
        },
        back: function () {
            $navigate.back();
        },
        eraseHistory: function () {
            $navigate.eraseHistory();
        }
    }
});

eventApp.factory('eventService', function ($rootScope, $q, $http, $filter, localStorageService, imageService, pushNotificationService, authService, initConf) {

    var LAST_LOAD_TIMESTAMP_KEY = 'last_load_timestamp';

    var myEvents, myEventsLength, events,eventsLength,eventLookbacks,eventLookbacksLength,
        kkEvents,kkEventsLength,kkEventLookbacks,kkEventLookbacksLength,
        erEvents,erEventsLength,erEventLookbacks,erEventLookbacksLength,
        lookbacksByYear,allEvents,allEventsLength,eventsLoading,eventsLoaded;

    var d = null;

    function resetEvents () {
        myEvents = [];
        myEventsLength = 0;
        events = [];
        eventsLength = 0;
        eventLookbacks = [];
        eventLookbacksLength = 0;
        kkEvents = [];
        kkEventsLength = 0;
        kkEventLookbacks = [];
        kkEventLookbacksLength = 0;
        erEvents = [];
        erEventsLength = 0;
        erEventLookbacks = [];
        erEventLookbacksLength = 0;

        lookbacksByYear = {
            'thisYear':     {'year':new Date().getFullYear(),    'events':[]},
            'lastYear':     {'year':new Date().getFullYear() - 1,'events':[]},
            'lastLastYear': {'year':new Date().getFullYear() - 2,'events':[]}
        };

        allEvents = {};
        allEventsLength = 0;

        eventsLoading = false;
        eventsLoaded = false;

    }
    resetEvents();

    function addTeaserImageURL (event) {
        // aus der puren ImageId einen komplette URL machen
        if(event.teaserImages != null) {
            event.eventTeaserImage = function() {
                return event.teaserImages[0];
/*
                imageService.loadImage(event.teaserImages[0]).then(function(imageSrcURL){
                    return imageSrcURL;
                });
*/
            };
/*
            imageService.loadImage(event.teaserImages[0]).then(function (imagePath) {
                event.eventTeaserImage = imagePath;
            });
*/
        }
    }
    function addToMyEvents (event) {
        if ( event.isFinished && (event.isSignedIn || event.isSignedInPending || event.isWarteliste || event.isSignedOff || event.isSignedOffPending) ){
            myEvents.push(event);
            myEventsLength++;
            return true;
        }
        return false;
    }

    function sortMyEvents () {
        myEvents = sortEventObjects (myEvents);
    }
    function sortEvents () {
        events = sortEventObjects (events);
    }
    function sortKKEvents () {
        kkEvents = sortEventObjects (kkEvents);
    }
    function sortEREvents () {
        kkEvents = sortEventObjects (kkEvents);
    }
    function sortEventObjects (events) {
        return $filter('orderBy')(events, function(e){ return e.startDateInMillis; }, false /*reverse*/);
    }
    function isER (event) {
        return event != null && event.type === 'reisen';
    }
    function addToLookbacksByYear(event) {
        var date = new Date(event.startDateInMillis);
        var year = date.getFullYear();
        if(lookbacksByYear.thisYear.year == year) {
            lookbacksByYear.thisYear.events.push(event);
        }
        if(lookbacksByYear.lastYear.year == year) {
            lookbacksByYear.lastYear.events.push(event);
        }
        if(lookbacksByYear.lastLastYear.year == year) {
            lookbacksByYear.lastLastYear.events.push(event);
        }
    }
    function sortLookbacksByYear() {
        angular.forEach(['thisYear','lastYear','lastLastYear'], function(yearIdent){
            lookbacksByYear[yearIdent].events = $filter('orderBy')(lookbacksByYear[yearIdent].events, function(e){ return e.startDateInMillis; }, true /*reverse*/);
        });
    }
    function replaceEventInArray(events, event) {
        angular.forEach(events, function(myEvent, index) {
            if(event.ident == myEvent.ident) {
                events[index] = event;
            }
        });
    }

    return {

        loadEvents : function () {

            if(eventsLoading) {
                return d.promise;
            }
            eventsLoading = true;

            d = $q.defer();
            $http({
                url: initConf.serverURL + "/main/json/event/getAll",
                method: "POST",
                data: {"pushregid":pushNotificationService.getPushRegId()}
            }).success(function(data, status, headers, config) {

              if (data.status == "OK")
              {
                  //eventsLength = data.events.length;
                  angular.forEach(data.events, function(value){

                      var isInMyEvents = addToMyEvents(value);

                      if (!isInMyEvents) {
                          if(isER(value)) {
                              erEvents.push(value);
                              erEventsLength++;
                          }
                          else {
                              events.push(value);
                              eventsLength++;
                          }

                      }
                      allEvents[value.ident] = value;

//                      addTeaserImageURL(value);
                  });
                  sortEvents(); // kommt "falsch" herum vom Server

                  //eventLookbacksLength = data.eventLookbacks.length;
                  angular.forEach(data.eventLookbacks, function(value){

                      if(isER(value)) {
                          erEventLookbacks.push(value);
                          erEventLookbacksLength++;
                      }
                      else {
                          eventLookbacks.push(value);
                          eventLookbacksLength++;
                      }
                      allEvents[value.ident] = value;

                      //addToLookbacksByYear(value);

//                      addTeaserImageURL(value);
                  });

                  kkEventsLength = data.kkEvents.length;
                  angular.forEach(data.kkEvents, function(value){

                      var isInMyEvents = addToMyEvents(value);

                      if (!isInMyEvents) kkEvents.push(value);
                      allEvents[value.ident] = value;

//                      addTeaserImageURL(value);
                  });
                  sortKKEvents();

                  kkEventLookbacksLength = data.kkEventLookbacks.length;
                  angular.forEach(data.kkEventLookbacks, function(value){
                        kkEventLookbacks.push(value);
                        allEvents[value.ident] = value;

                        //addToLookbacksByYear(value);

    //                      addTeaserImageURL(value);
                  });

                  // jetzt haben wir alle zusammen fuer myEvents, also koennen wir die Liste jetzt sortieren
                  sortMyEvents();

                  //sortLookbacksByYear();

                  allEventsLength = eventsLength + eventLookbacksLength
                                    + kkEventsLength + kkEventLookbacksLength
                                    + erEventsLength + erEventLookbacksLength;

                  localStorageService.add(LAST_LOAD_TIMESTAMP_KEY, (new Date()).getTime());
                  eventsLoaded = true;
                  eventsLoading = false;

                  d.resolve();
              }
              else
              {
                  eventsLoading = false;

                  var reason = "Unser Server hat leider ein Problem. Versuchen Sie es später noch einmal.";
                  d.reject(reason);
              }
            }).error(function(data, status, headers, config) {

                eventsLoading = false;

                var reason = "Verbindungsprobleme zu unserem Server. (Status " + status+")";
                d.reject(reason);
            });

            return d.promise;
        },
        getEventsLoaded : function () {
            if(!eventsLoaded) return false;

            // Sind die Daten aelter als initConf.eventReloadIntervall?
            var lastLoadTimestamp = localStorageService.get(LAST_LOAD_TIMESTAMP_KEY); // kommt als String
            var now = (new Date()).getTime();
            if( lastLoadTimestamp == null || (parseInt(lastLoadTimestamp) + initConf.eventReloadIntervall) < now) {

                this.resetEvents();
                return false;
            }

            return true;
        },
        getMyEvents : function () {
          return myEvents;
        },
        getMyEventsLength : function () {
          return myEventsLength;
        },
        getEvents : function () {
          return events;
        },
        getEventsLength : function () {
          return eventsLength;
        },
        getEventLookbacks : function () {
          return eventLookbacks;
        },
        getEventLookbacksLength : function () {
          return eventLookbacksLength;
        },
        getKKEvents : function () {
          return kkEvents;
        },
        getKKEventsLength : function () {
          return kkEventsLength;
        },
        getKKEventLookbacks : function () {
          return kkEventLookbacks;
        },
        getKKEventLookbacksLength : function () {
          return kkEventLookbacksLength;
        },
        getEREvents : function () {
          return erEvents;
        },
        getEREventsLength : function () {
          return erEventsLength;
        },
        getEREventLookbacks : function () {
          return erEventLookbacks;
        },
        getEREventLookbacksLength : function () {
          return erEventLookbacksLength;
        },
        isReise : function (event) {
          return isER(event);
        },
        getLookbacksByYear : function () {
          return lookbacksByYear;
        },


        getAllByType : function (type) {
            switch (type) {
                case 'allEvents':
                  return erEvents.concat(events).concat(kkEvents); // TODO evtl. muessen wir noch nach Datum sortieren
                  break;
                case 'myEvents':
                  return myEvents;
                  break;
                case 'lookbacks':
                  return erEventLookbacks.concat(eventLookbacks).concat(kkEventLookbacks); // TODO evtl. muessen wir noch nach Datum sortieren
                  break;
                default:
                    return [];
            }
        },
        getByIdent : function (eventIdent) {
            var d = $q.defer();

            if(this.getEventsLoaded() && allEvents[eventIdent] != null) {

                d.resolve(allEvents[eventIdent]);

            } else {
                this.loadEvents().then(function() {
                    d.resolve(allEvents[eventIdent]);
                },
                function(reason){
                    d.reject(reason);
                });
            }

            return d.promise;
        },
        getByWebcode : function (webcode) {
            var d = $q.defer();
            function findCode() {
                var eventIdent = null;
                angular.forEach(allEvents, function(event){
                    if(webcode!=null &&  event.webcode != null && webcode.toLowerCase() == event.webcode.toLowerCase()) {
                        eventIdent = event.ident;
                    }
                });
                return eventIdent;
            }
            if(this.getEventsLoaded()) {

                d.resolve(findCode());

            } else {
                this.loadEvents().then(function() {
                    d.resolve(findCode());
                },
                function(reason){
                    d.reject(reason);
                });
            }

            return d.promise;
        },
        setRegistrationStatusToSignedInPending : function (eventIdent) {
            this.getByIdent(eventIdent).then(function(event){
                if(event.isInvited || event.isSignedOff) {

                    event.isSignedInPending = true;
                    event.isInvited = false;
                    event.isSignedOff = false;

                    allEvents[eventIdent] = event;
                    if(event.isInvited) {
                        myEvents.push(event);
                        sortMyEvents();
                    }
                    if(event.isSignedOff) {
                        // das angepasste Event in myEvents ersetzen
                        replaceEventInArray(myEvents, event);
                    }
                }
            });
        },
        setRegistrationStatusToSignedOffPending : function (eventIdent) {
            this.getByIdent(eventIdent).then(function(event){
                if(event.isSignedIn) {

                    event.isSignedOffPending = true;
                    event.isSignedIn = false;
                    allEvents[eventIdent] = event;

                    // das angepasste Event in myEvents ersetzen
                    replaceEventInArray(myEvents, event);

                }
            });
        },

        /* nur fuer debug */
        getAllEventsLength : function () {
            return allEventsLength;
        },
        resetEvents : function () {
            myEvents = [];
            myEventsLength = 0;
            events = [];
            eventsLength = 0;
            eventLookbacks = [];
            eventLookbacksLength = 0;
            kkEvents = [];
            kkEventsLength = 0;
            kkEventLookbacks = [];
            kkEventLookbacksLength = 0;
            erEvents = [];
            erEventsLength = 0;
            erEventLookbacks = [];
            erEventLookbacksLength = 0;

            lookbacksByYear = {
                'thisYear':     {'year':new Date().getFullYear(),    'events':[]},
                'lastYear':     {'year':new Date().getFullYear() - 1,'events':[]},
                'lastLastYear': {'year':new Date().getFullYear() - 2,'events':[]}
            };

            allEvents = {};
            allEventsLength = 0;

            eventsLoaded = false;

        }
    }
});

eventApp.factory('eventQuestionAnswerService', function ($rootScope, $q, $http, $log, eventService, initConf) {

    var allQuestions = {};

    var questionsLoading = false;

    var d = null;
    return {

        loadQuestionAnswers : function (eventIdent) {
            if(questionsLoading) {
                return d.promise;
            }
            questionsLoading = true;

            d = $q.defer();
            var now = (new Date()).getTime();
            if (allQuestions[eventIdent] != null && parseInt(allQuestions[eventIdent].loadDate) + initConf.eventQuestionsReloadIntervall > now) {

                questionsLoading = false;
                d.resolve(allQuestions[eventIdent]);

            } else {
                $http({
                    url: initConf.serverURL + "/main/json/eqa/questions",
                    method: "POST",
                    data: {"id":eventIdent}
                }).success(function(data, status, headers, config) {

                    if (data.status == "OK")
                    {
                        data.loadDate = (new Date()).getTime();

                        allQuestions[eventIdent] = data;

                        questionsLoading = false;

                        d.resolve(data);
                    }
                    else
                    {
                        questionsLoading = false;

                        var reason = "Unser Server hat leider ein Problem. Versuchen Sie es später noch einmal.";
                        d.reject(reason);
                    }
                }).error(function(data, status, headers, config) {
                    questionsLoading = false;

                    var reason = "Verbindungsprobleme zu unserem Server. (Status " + status+")";
                    d.reject(reason);
                });
            }
            return d.promise;
        },

        getByEventIdent : function (eventIdent) {
            return this.loadQuestionAnswers(eventIdent);
        },

        sendAnswers : function (eventIdent, answers) {
            var d = $q.defer();

            $http({
                url: initConf.serverURL + "/main/json/eqa/answers",
                method: "POST",
                data: angular.extend({"id":eventIdent}, answers)
            }).success(function(data, status, headers, config) {

                if (data.status == "OK")
                {
                    eventService.setRegistrationStatusToSignedInPending(eventIdent);

                    d.resolve(data);
                }
                else
                {
                    var reason = data.reason || "Unser Server hat leider ein Problem. Versuchen Sie es später noch einmal.";
                    d.reject(reason);
                }
            }).error(function(data, status, headers, config) {
                var reason = "Verbindungsprobleme zu unserem Server. (Status " + status+")";
                d.reject(reason);
            });

            return d.promise;
        },

        sendSignOff : function (eventIdent) {
                    var d = $q.defer();

                    $http({
                        url: initConf.serverURL + "/main/json/eqa/signOff",
                        method: "POST",
                        data: {"id":eventIdent}
                    }).success(function(data, status, headers, config) {

                        if (data.status == "OK")
                        {
                            eventService.setRegistrationStatusToSignedOffPending(eventIdent);

                            d.resolve(data);
                        }
                        else
                        {
                            var reason = data.reason || "Unser Server hat leider ein Problem. Versuchen Sie es später noch einmal.";
                            d.reject(reason);
                        }
                    }).error(function(data, status, headers, config) {
                                var reason = "Verbindungsprobleme zu unserem Server. (Status " + status+")";
                        d.reject(reason);
                    });

                    return d.promise;
                }
    }
});

eventApp.factory('contentService', function ($rootScope, $q, $http, initConf) {

    var allContents = {};

    var contentLoading = false;

    var d = null;
    return {

        loadContent : function (ident) {

            if(contentLoading) {
                return d.promise;
            }
            contentLoading = true;

            d = $q.defer();
            var now = (new Date()).getTime();
            if (allContents[ident] != null && parseInt(allContents[ident].loadDate) + initConf.contentReloadIntervall > now) {

                contentLoading = false;
                d.resolve(allContents[ident]);

            } else {

                $http({
                    url: initConf.serverURL + "/main/json/app/content",
                    method: "POST",
                    data: {"id":ident}
                }).success(function(data, status, headers, config) {

                  if (data.status == "OK")
                  {
                      data.loadDate = (new Date()).getTime();
                      allContents[ident] = data;

                      contentLoading = false;

                      d.resolve(data);
                  }
                  else
                  {
                      contentLoading = false;

                      var reason = "Unser Server hat leider ein Problem. Versuchen Sie es später noch einmal.";
                      d.reject(reason);
                  }
                }).error(function(data, status, headers, config) {
                    contentLoading = false;

                    var reason = "Verbindungsprobleme zu unserem Server. (Status " + status+")";
                    d.reject(reason);
                });

            }
            return d.promise;
        },

        getByIdent : function (ident) {

            return this.loadContent(ident);
        }
    }
});

eventApp.factory('imageGalleryService', function ($rootScope, $q, $http, imageTokenService, initConf) {

    var galleries = {};

    var requestLoading = false;

    var d = null;
    return {

        loadImageGallery : function (ident, page) {

            if(requestLoading) {
                return d.promise;
            }
            requestLoading = true;

            d = $q.defer();
//            var now = (new Date()).getTime();
//            if (allContents[ident] != null && parseInt(allContents[ident].loadDate) + $rootScope.contentReloadIntervall > now) {
//
//                contentLoading = false;
//                d.resolve(allContents[ident]);
//
//            } else {
                var _page = page || 1;
                $http({
                    url: initConf.serverURL + "/main/json/imageGallery/showImageGallery",
                    method: "POST",
                    data: {"eventid":ident,"page":_page}
                }).success(function(data, status, headers, config) {

                  if (data.status == "OK")
                  {
                      data.loadDate = (new Date()).getTime();

                      angular.forEach(data.images, function(image){
                          imageTokenService.getImageToken().then(function(imageToken){
                              image.src = initConf.serverURL+'/main/json/imageGallery/getThumbnailImage?'+imageToken.name+'='+imageToken.value+'&referer=mobile'
                                                              +'&gid='+data.imageGalleryName+'&name='+image.name;
                          }, function(){});

                          imageTokenService.getImageToken().then(function(imageToken){
                              image.largeSrc = initConf.serverURL+'/main/json/imageGallery/getOnlineImage?'+imageToken.name+'='+imageToken.value+'&referer=mobile'
                                                                     +'&gid='+data.imageGalleryName+'&name='+image.name;
                          }, function(){});


                      });

                      galleries[ident] = data;

                      requestLoading = false;

                      d.resolve(data);
                  }
                  else
                  {
                      requestLoading = false;

                      var reason = "Unser Server hat leider ein Problem. Versuchen Sie es später noch einmal.";
                      d.reject(reason);
                  }
                }).error(function(data, status, headers, config) {
                    requestLoading = false;

                    var reason = "Verbindungsprobleme zu unserem Server. (Status " + status+")";
                    d.reject(reason);
                });

//            }
            return d.promise;
        }
    }
});

eventApp.factory('kamInfoService', function ($rootScope, $q, $http, localStorageService, initConf) {

    var KAM_INFO_KEY = "kamInfos";
    var KAM_INFO_TIMESTAMP_KEY = "kamInfosLastLoadTimestamp";

    var kamInfoLoading = false;

    var d = null;

    return {
        getKAMInfos : function () {

            if(kamInfoLoading) {
                return d.promise;
            }
            kamInfoLoading = true;

            d = $q.defer();
            var now = (new Date()).getTime();
            if (localStorageService.get(KAM_INFO_KEY) != null
                    && parseInt(localStorageService.get(KAM_INFO_TIMESTAMP_KEY)) + initConf.kamReloadIntervall > now) {

                kamInfoLoading = false;
                d.resolve(localStorageService.get(KAM_INFO_KEY));

            } else {

                $http({
                    url: initConf.serverURL + "/main/json/getKAMInfos",
                    method: "POST",
                    data: {}
                }).success(function(data, status, headers, config) {

                      localStorageService.add(KAM_INFO_KEY, data);
                      localStorageService.add(KAM_INFO_TIMESTAMP_KEY, (new Date()).getTime());

                      kamInfoLoading = false;

                      d.resolve(data);

                }).error(function(data, status, headers, config) {
                    kamInfoLoading = false;

                    var reason = "Verbindungsprobleme zu unserem Server. (Status " + status+")";
                    d.reject(reason);
                });

            }
            return d.promise;
        },
        resetKAMInfos : function () {
            localStorageService.remove(KAM_INFO_KEY);
        },
        hasKAMInfos : function () {
            return localStorageService.get(KAM_INFO_KEY) != null;
        }
    }
});
eventApp.factory('feedbackService', function ($rootScope, $q, $http, localStorageService, initConf) {

    var feedbackSending = false;

    var d = null;

    return {
        sendEventFeedback : function (eventIdent, msg) {

            if(feedbackSending) {
                return d.promise;
            }
            feedbackSending = true;

            d = $q.defer();

            $http({
                url: initConf.serverURL + "/main/json/sendEventFeedback",
                method: "POST",
                data: {"eventId":eventIdent, "text": msg, "fromPage": "zu einer Veranstaltung"}
            }).success(function(data, status, headers, config) {

                  feedbackSending = false;

                  d.resolve(data);

            }).error(function(data, status, headers, config) {
                feedbackSending = false;
                var reason = 'Keine Verbindung zum Server.';
                d.reject(reason);
            });

            return d.promise;
        },
        sendEventRating : function (eventIdent, answers) {

            if(feedbackSending) {
                return d.promise;
            }
            feedbackSending = true;

            d = $q.defer();

            $http({
                url: initConf.serverURL + "/main/json/event/rate",
                method: "POST",
                data: angular.extend({"eventId":eventIdent}, answers)
            }).success(function(data, status, headers, config) {

                  feedbackSending = false;

                  d.resolve(data);

            }).error(function(data, status, headers, config) {
                feedbackSending = false;
                var reason = 'Keine Verbindung zum Server.';
                d.reject(reason);
            });

            return d.promise;
        }
    }
});

eventApp.factory('imageService', function ($rootScope, $q, $log, phonegapReady, imageTokenService, initConf) {
    var DATADIR = null;
    var knownfiles = {};

    var serviceInitialised = false;

    function onFSSuccess (fileSystem, d) {
        //var base = "Android/data/de.deutschepost.forumgelb";
        //var base = "de.deutschepost.forumgelb";
//        var base = "de.akquinet.forumgelb";
        var base = "downloadedImages";
        fileSystem.root.getDirectory(base, {create:true},
                                     function(dir){
                                        gotDir(dir,d);
                                     } /* success*/,
                                     function(err) { /* error */
                                         onError(err, d);
                                     }
        );

    };

    function onError (e, d) {
        //alert('Error: '+JSON.stringify(e));
        d.reject('Fehler beim Zugriff auf das Filesystem');
    }

    //The directory entry callback
    function gotDir (dir, d){
        DATADIR = dir;
        var reader = DATADIR.createReader();
        reader.readEntries(
            function(entries){
                //alert("The dir has "+entries.length+" entries. \n DATADIR = "+DATADIR.toURL());
                for (var i=0; i<entries.length; i++) {
                    knownfiles[entries[i].name] = entries[i].toURL();
                }

                serviceInitialised = true;
                d.resolve();
            },
            function(err) { onError(err, d); });
    }

    var serviceInitDefer = null;
    var serviceInitializingInProgress = false;
    function isServiceInitialised() {

        if(serviceInitializingInProgress) return serviceInitDefer.promise;

        serviceInitDefer = $q.defer();
        serviceInitializingInProgress = true;

        if(!serviceInitialised) {

             if(window.requestFileSystem != null) {
                window.requestFileSystem(LocalFileSystem.PERSISTENT, 0,
                                  function(fileSystem){
                                    onFSSuccess(fileSystem, serviceInitDefer);
                                  },
                                  function(err) { onError(err, serviceInitDefer); });
            } else {
                onError('', serviceInitDefer);
            }
        }
        else
        {
            serviceInitDefer.resolve();
        }

        return serviceInitDefer.promise;
    }
    function handle (ident, asWebURL) {

        var d = $q.defer();

        //TODO hier wird immer ein Token verbraucht, selbst wenn wir die Bilder aus dem Filesystem holen -> kann man das Optimieren?!
        imageTokenService.getImageToken().then(function(imageToken) {

            var url = initConf.serverURL + "/main/json/teaserImage?" + imageToken.name + "=" + imageToken.value+ "&referer=mobile&id=" + ident;

            if (asWebURL) {

                d.resolve(url);

            } else {

                var imgPath = knownfiles[ident];
                if (imgPath != null) {

                    d.resolve(imgPath);

                } else {

                    var ft = new FileTransfer();
                    var dlPath = DATADIR.toURL() + "/" + ident;

                    ft.download(url,
                            dlPath,
                            function (img) {
                                knownfiles[ident] = img.toURL();
                                // TODO: sma: vorlaeufiger workaround...
                                d.resolve(img.toURL());
                            },
                            function (e) {
                                d.reject(e);
                            },
                            true /*trustAllHosts*/);

                    // TODO sma Workaround!!! gleich nachdem der Download angestossen wurde, mit der Web-URL aufloesen
                    // wenn die App spaeter mal auf die "Platte" schaut und die DAtei findet, wird diese genommen, solange
                    // wird hier die Web-URL geliefert...
                    //d.resolve(url);
                }
            }

        }, function(){
            // error
            var msg = 'kein ImageToken erhalten';
            $log.debug(msg);
            d.reject(msg);
        });

        return d.promise;

    }
    return {
        loadImage : function (ident){
              var d = $q.defer();
              isServiceInitialised().then(
                    function(){
                        handle(ident).then(
                            function(imgPath){
                                 d.resolve(imgPath);
                            },
                            function(e){
                                 d.reject(e);
                            }
                        );
                    },
                    function(){
                        //d.reject('error beim imageService initialisieren');

                        // wenn der Zugriff auf das Filesystem nicht klappt, dann den alten Weg mit webURL gehen
                        handle(ident, true /* mit webURL*/).then(
                                function(imgPath){
                                     d.resolve(imgPath);
                                },
                                function(e){
                                     d.reject(e);
                                }
                        );
                    }
              );
              return d.promise;

        },
        getByIdent : function (ident) {

            return this.loadImage(ident);
        }

    }
});

eventApp.factory('pushNotificationService', function($rootScope, localStorageService, $q, $http){

    var AUTH_PUSH_REG_ID = 'pushRegId';

    return {
        pushNotififationSuccessHandler: function(result) {
            alert('Callback Success! Result = '+result)
        },

        pushNotififationErrorHandler:function(error) {
            alert(error);
        },
        getPushRegId:function() {
          return localStorageService.get (AUTH_PUSH_REG_ID);
        },
        onNotificationGCM: function(e) {
                switch( e.event )
                {
                    case 'registered':
                        if ( e.regid.length > 0 )
                        {
                            //console.log("Regid " + e.regid);
                            alert('registration id = '+e.regid);
                            localStorageService.add (AUTH_PUSH_REG_ID, e.regid);
                            alert('!!!:' + getPushRegId);
                        }
                    break;

                    case 'message':
                      // this is the actual push notification. its format depends on the data model from the push server
                      alert('message = '+e.message+' msgcnt = '+e.msgcnt);
                    break;

                    case 'error':
                      alert('GCM error = '+e.msg);
                    break;

                    default:
                      alert('An unknown GCM event has occurred');
                      break;
                }
            }
      }
});

eventApp.factory('notificationService', function ($rootScope, phonegapReady) {

        return {
            alert: phonegapReady(function (message, alertCallback, title, buttonName) {
                if(navigator.notification) {
                    navigator.notification.alert(message, function () {
                        var that = this,
                                args = arguments;

                        $rootScope.$apply(function () {
                            alertCallback.apply(that, args);
                        });
                    }, title, buttonName);
                } else {
                    alert(title + '\n'+message);
                }
            }),
            confirm: phonegapReady(function (message, confirmCallback, title, buttonLabels) {
                if(navigator.notification) {
                    navigator.notification.confirm(message, function () {
                        var that = this,
                                args = arguments;

                        $rootScope.$apply(function () {
                            confirmCallback.apply(that, args);
                        });
                    }, title, buttonLabels);
                } else {
                    var iWant = confirm(title+'\n'+message+'\n'+buttonLabels.join(','));
                    var num = iWant ? 1 : 0;
                    var that = this;
                    confirmCallback.apply(that, [num]);
                }
            }),
            beep: function (times) {
                navigator.notification.beep(times);
            },
            vibrate: function (milliseconds) {
                navigator.notification.vibrate(milliseconds);
            }
        };
});

